#version 330 core
#extension GL_ARB_explicit_uniform_location : enable
#extension GL_ARB_separate_shader_objects : enable

in vec2 fragUV;

layout(location = 1) uniform sampler2D color;
layout(location = 2) uniform sampler2D normal;
layout(location = 3) uniform sampler2D depth;

layout(location = 4) uniform int resolution[2];

uniform bool edgeDetection;
uniform bool depthOfField;

out vec4 outColor;

void main()
{
	float dx = 1.0/resolution[0];
	float dy = 1.0/resolution[1];
	outColor = texture(color, fragUV);

	float n = 0.0;

	if (edgeDetection)
		{
		vec3 middle = texture(normal, fragUV).xyz;
		vec3 right = texture(normal, fragUV + vec2(dx, 0.0)).xyz;
		vec3 left = texture(normal, fragUV + vec2(-dx, 0.0)).xyz;
		vec3 top = texture(normal, fragUV + vec2(0.0, dy)).xyz;
		vec3 bottom = texture(normal, fragUV + vec2(0.0, -dy)).xyz;
		vec3 topright = texture(normal, fragUV + vec2(dx, dy)).xyz;
		vec3 topleft = texture(normal, fragUV + vec2(-dx, dy)).xyz;
		vec3 bottomright = texture(normal, fragUV + vec2(dx, -dy)).xyz;
		vec3 bottomleft = texture(normal, fragUV + vec2(-dx, -dy)).xyz;

		vec3 result1 = 3*topleft + 6*top + 3*topright - 3*bottomleft - 6*bottom - 3*bottomright;
		vec3 result2 = 3*topleft + 6*left + 3*bottomleft - 3*topright - 6*right - 3*bottomright;

		vec3 gradient = sqrt(pow(result1, vec3(2.0, 2.0, 2.0)) + pow(result2, vec3(2.0, 2.0, 2.0)));
		n = max(n, gradient.x);
		n = max(n, gradient.y);
		n = max(n, gradient.z);
		n = (gradient.x + gradient.y + gradient.z) /3;

		if (n > 4.0) outColor = vec4(0.0, 0.0, 0.0, 1.0);
	}

	if (depthOfField && !(n > 4.0))
		{
		vec3 middle = texture(color, fragUV).xyz;
		vec3 right = texture(color, fragUV + vec2(dx, 0.0)).xyz;
		vec3 left = texture(color, fragUV + vec2(-dx, 0.0)).xyz;
		vec3 top = texture(color, fragUV + vec2(0.0, dy)).xyz;
		vec3 bottom = texture(color, fragUV + vec2(0.0, -dy)).xyz;
		vec3 topright = texture(color, fragUV + vec2(dx, dy)).xyz;
		vec3 topleft = texture(color, fragUV + vec2(-dx, dy)).xyz;
		vec3 bottomright = texture(color, fragUV + vec2(dx, -dy)).xyz;
		vec3 bottomleft = texture(color, fragUV + vec2(-dx, -dy)).xyz;

		vec3 result = topleft + 2*top + topright + 2*left + 4*middle + 2*right + bottomleft + 2*bottom + bottomright;
		result = result / vec3(16, 16, 16);

		float dof = texture(depth, fragUV).x;
		outColor = vec4(vec3(dof, dof, dof) * result + vec3(1-dof, 1-dof, 1-dof) * middle, texture(color, fragUV).z);
	}
}